@@ -6,13 +6,14 @@
 /*                 Christophe Tribes           - Polytechnique Montreal            */
 /*                                                                                 */
 /*  The copyright of NOMAD - version 4.0.0 is owned by                             */
+/*                 Charles Audet               - Polytechnique Montreal            */
 /*                 Sebastien Le Digabel        - Polytechnique Montreal            */
 /*                 Viviane Rochon Montplaisir  - Polytechnique Montreal            */
 /*                 Christophe Tribes           - Polytechnique Montreal            */
 /*                                                                                 */
-/*  NOMAD v4 has been funded by Rio Tinto, Hydro-Québec, NSERC (Natural Science    */
-/*  and Engineering Research Council of Canada), INOVEE (Innovation en Energie     */
-/*  Electrique and IVADO (The Institute for Data Valorization)                     */
+/*  NOMAD v4 has been funded by Rio Tinto, Hydro-Québec, NSERC (Natural            */
+/*  Sciences and Engineering Research Council of Canada), InnovÉÉ (Innovation      */
+/*  en Énergie Électrique) and IVADO (The Institute for Data Valorization)         */
 /*                                                                                 */
 /*  NOMAD v3 was created and developed by Charles Audet, Sebastien Le Digabel,     */
 /*  Christophe Tribes and Viviane Rochon Montplaisir and was funded by AFOSR       */
@@ -26,8 +27,6 @@
 /*    Polytechnique Montreal - GERAD                                               */
 /*    C.P. 6079, Succ. Centre-ville, Montreal (Quebec) H3C 3A7 Canada              */
 /*    e-mail: nomad@gerad.ca                                                       */
-/*    phone : 1-514-340-6053 #6928                                                 */
-/*    fax   : 1-514-340-5665                                                       */
 /*                                                                                 */
 /*  This program is free software: you can redistribute it and/or modify it        */
 /*  under the terms of the GNU Lesser General Public License as published by       */
@@ -55,10 +54,11 @@
 #ifndef __NOMAD400_CACHESET__
 #define __NOMAD400_CACHESET__
 
+#ifdef _OPENMP
+#include <omp.h>
+#endif  // _OPENMP
 #include "../Cache/CacheBase.hpp"
-#include "../Eval/Eval.hpp"
 #include "../Eval/EvalPoint.hpp"
-#include "../Math/Point.hpp"
 
 #include "../nomad_nsbegin.hpp"
 
@@ -76,7 +76,7 @@ private:
     static omp_lock_t _cacheLock;
 #endif // _OPENMP
 
-    static BBOutputTypeList _bbOutputType; ///< Corresponds to parameter BB_OUTPUT_TYPE used for this cach
+    static BBOutputTypeList _bbOutputType; ///< Corresponds to parameter BB_OUTPUT_TYPE used for this cache
 
     EvalPointSet _cache;  ///< The set of points that constitutes the cache.
 
@@ -130,15 +130,17 @@ public:
 
     /// Get eval point at point x from the cache (there can be only one in CacheSet).
     /**
-     \param x           The point to find               -- \b IN.
-     \param evalPoint   The copy of the data in cache   -- \b OUT.
+     \param x           The point to find                   -- \b IN.
+     \param evalPoint   The copy of the data in cache       -- \b OUT.
+     \param evalType    If not UNDEFINED, wait for the point to be evaluated for this EvalType. -- \b IN.
      \return            An integer: 1 if found, 0 otherwise
      */
-    size_t find(const Point & x,
-                EvalPoint &evalPoint) const override;
+    size_t find(const Point & x, EvalPoint &evalPoint,
+                const EvalType evalType = EvalType::UNDEFINED) const override;
 
     /// Insert evalPoint in cache.
     /**
+     * evalPoint's tag (mutable) is updated.
      * Return a boolean indicating if we should eval this point. \n
      * If insertion worked, the point was not in the cache before. Return true. \n
      * If insertion did not work, the point was in the cache before. \n
@@ -167,14 +169,14 @@ public:
      *
      * All eval points for which eval is inferior to refeval.
 
-     \param refeval         The point to find                                              -- \b IN.
-     \param comp            The comparison function                                        -- \b IN.
+     \param refeval                The point to find                                              -- \b IN.
+     \param comp                       The comparison function                                        -- \b IN.
      \param evalPointList   The eval points that verify comp()==true returned in a list    -- \b OUT.
-     \param evalType        Which eval (Blackbox or Surrogate) of the EvalPoint to look at  -- \b IN.
-     \return                The number of eval points found.
+     \param evalType              Which eval (Blackbox or Surrogate) of the EvalPoint to look at  -- \b IN.
+     \return                 The number of eval points found.
      */
     size_t find(const Eval &refeval,
-             bool (*comp)(const Eval&, const Eval&),
+             std::function<bool(const Eval&, const Eval&)> comp,
              std::vector<EvalPoint> &evalPointList,
              const EvalType& evalType = EvalType::BB) const override;
 
@@ -182,31 +184,35 @@ public:
      * \brief Get best eval points, using comp().
      *
      * Only the points with eval status EVAL_OK are considered.
-     \param comp             The comparison function                                    -- \b IN.
+     \param comp                        The comparison function                                    -- \b IN.
      \param evalPointList    The best eval points that verify comp()==true in a list    -- \b OUT.
-     \param findFeas         Flag to find feasible points                               -- \b IN.
-     \param hMax             Maximum acceptable value for h, when findFeas is false     -- \b IN.
+     \param findFeas               Flag to find feasible points                               -- \b IN.
+     \param hMax                        Maximum acceptable value for h, when findFeas is false     -- \b IN.
      \param fixedVariable    Searching for a subproblem defined by this point -- \b IN.
-     \param evalType         Which Eval (Blackbox or Surrogate) of the EvalPoint to look at  -- \b IN.
+     \param evalType               Which Eval (Blackbox or Surrogate) of the EvalPoint to look at  -- \b IN.
+     \param refeval                 The point of reference                                              -- \b IN.
      \return                 The number of eval points found.
      */
-    virtual size_t findBest(bool (*comp)(const Eval&, const Eval&),
+    virtual size_t findBest(std::function<bool(const Eval&, const Eval&)> comp,
                      std::vector<EvalPoint> &evalPointList,
                      const bool findFeas,
                      const Double& hMax,
                      const Point& fixedVariable,
-                     const EvalType& evalType) const override;
+                     const EvalType& evalType,
+                     const Eval* refeval) const override;
 
     /// Find best feasible points, using operator<.
     /**
      \param evalPointList   The best feasible eval points in a list  -- \b OUT.
-     \param evalType        Which eval (Blackbox or Surrogate) of the EvalPoint to look at  -- \b IN.
      \param fixedVariable   Searching for a subproblem defined by this point -- \b IN.
-     \return                The number of eval points found.
+     \param evalType              Which eval (Blackbox or Surrogate) of the EvalPoint to look at  -- \b IN.
+     \param refeval                 The single best feasible eval point -- \b OUT.
+     \return                 The number of eval points found.
      */
     virtual size_t findBestFeas(std::vector<EvalPoint> &evalPointList,
                                 const Point& fixedVariable,
-                                const EvalType& evalType) const override;
+                                const EvalType& evalType,
+                                const Eval* refeval) const override;
 
     /// Test if cache contains a feasible point.
     /**
@@ -217,38 +223,55 @@ public:
     /// Find best infeasible points, with h <= hMax, using operator<.
     /**
      \param evalPointList   The best infeasible eval points in a list   -- \b OUT.
-     \param hMax            Select a point if h <= hMax                 -- \b IN.
+     \param hMax                       Select a point if h <= hMax                 -- \b IN.
      \param fixedVariable   The fixed variables have a fixed value      -- \b IN.
-     \param evalType        Which eval (Blackbox or Surrogate) of the EvalPoint to look at  -- \b IN.
+     \param evalType              Which eval (Blackbox or Surrogate) of the EvalPoint to look at  -- \b IN.
+     \param refeval               The single best infeasible eval point -- \b OUT.
      \return                The number of eval points found.
      */
     virtual size_t findBestInf(std::vector<EvalPoint> &evalPointList,
                             const Double& hMax,
                             const Point& fixedVariable,
-                            const EvalType& evalType) const override;
+                            const EvalType& evalType,
+                            const Eval* refeval) const override;
 
-    /// Get all eval points within a distance of point X.
+    /// Get all eval points verifying a criterion function with respect to point X. The criterion function can be a measure of distance to X.
     /**
-     \param X                The point of reference                              -- \b IN.
-     \param distance         The distance to the point of reference              -- \b IN.
+
+     \param X                               The point of reference                              -- \b IN.
+     \param crit                        The criterion function                              -- \b IN.
      \param evalPointList    The eval points within the prescribed distance of X -- \b OUT.
      \param maxEvalPoints    The maximum number of points to select              -- \b IN.
      \return                 The number of eval points found.
      */
-    size_t find(Point X,
-                Double distance,
+    virtual size_t find(const Point & X,
+                std::function<bool(const Point&, const EvalPoint &)> crit,
                 std::vector<EvalPoint> &evalPointList,
                 int maxEvalPoints = 0) const override;
 
-    /// \brief Find using criteria.
+    /// \brief Find using custom criteria.
     /**
      All the points for which crit() return true are put in evalPointList.
 
-     \param crit             The criteria function                               -- \b IN.
+     \param crit                        The criteria function                               -- \b IN.
      \param evalPointList    The eval points within the prescribed distance of X -- \b OUT.
      \return                 The number of eval points found.
      */
-    virtual size_t find(bool (*crit)(const EvalPoint&),
+    virtual size_t find(std::function<bool(const EvalPoint&)> crit,
+                        std::vector<EvalPoint> &evalPointList) const override;
+
+
+    /// \brief Find using custom criteria  and distance to a point.
+    /**
+     All the points for which the two crit() functions return true are put in evalPointList.
+
+     \param crit1                      The first criteria function                                    -- \b IN.
+     \param crit2                      The second criteria function                               -- \b IN.
+     \param evalPointList    The eval points within the prescribed distance of X -- \b OUT.
+     \return                 The number of eval points found.
+     */
+    virtual size_t find(std::function<bool(const EvalPoint&)> crit1,
+                        std::function<bool(const EvalPoint&)> crit2,
                         std::vector<EvalPoint> &evalPointList) const override;
 
 
@@ -271,7 +294,7 @@ public:
     bool clear() override;
 
     /// Clear all sgte evaluations from the cache
-    void clearSgte() override;
+    void clearSgte(const int mainThreadNum) override;
 
     /** Purge the cache to get under MAX_CACHE_SIZE.
      * \todo The current implementation is naïve. It could be more sophisticated.
@@ -285,7 +308,7 @@ public:
     bool read() override;
 
     /// Display all points in cache.
-    std::ostream& displayAll(std::ostream& os) const;
+    std::string displayAll() const override;
 
     /** Display only EvalPoints that have an eval.
      * This method is used to write the cache file.
@@ -297,12 +320,16 @@ public:
     /// Compute the mean f.
     /**
      \param mean       The eval point to update -- \b OUT.
-     \return           The number of EvalPoints for which f is defined.
+     \return        The number of EvalPoints for which f is defined.
      */
     size_t computeMeanF(Double &mean) const override;
 
     /// Call function func() on all EvalPoint in cache.
     void processOnAllPoints(void (*func)(EvalPoint&)) override;
+    /// Call function func() on all EvalPoint in cache for Evals that were generated by mainThreadNum.
+    void processOnAllPoints(void (*func)(EvalPoint&), int mainThreadNum) override;
+
+    void deleteSgteOnly(const int mainThreadNum) override;
 
     /// Recompute F and H on a cache point.
     /**
