@@ -359,6 +359,7 @@ static PyObject* minimize(PyObject* /* dummy */, PyObject* args, PyObject* kwds)
             false, /* have DISPLAY_ALL_EVAL */
             false, /* have DIMENSION */
             false  /* have SEED */ };
+
         if (kwds) {
             PyObject* items = PyDict_Items(kwds);
             for (Py_ssize_t i = 0; i < PyList_GET_SIZE(items); ++i) {
@@ -370,22 +371,29 @@ static PyObject* minimize(PyObject* /* dummy */, PyObject* args, PyObject* kwds)
                     break;
                 }
 
-                if (strcmp(key, "MAX_BB_EVAL") == 0 || strcmp(key, "MAX_EVAL") == 0) {
-                    long budget = PyInt_AsLong(PyTuple_GET_ITEM(pair, 1));
-                    if (budget == -1 && PyErr_Occurred()) {
+                if (strcmp(key, "MAX_BB_EVAL") == 0) {
+                    size_t budget = PyLong_AsSize_t(PyTuple_GET_ITEM(pair, 1));
+                    if (budget == (size_t)-1 && PyErr_Occurred()) {
                         options_ok[0] = false;
                         break;
                     }
-                    params->setAttributeValue("MAX_BB_EVAL", (int)budget);
+                    params->setAttributeValue<size_t>(key, (size_t)budget);
 
                 // have to set MAX_EVAL along side MAX_BB_EVAL, unless overridden in options
-                    if (strcmp(key, "MAX_BB_EVAL") == 0) {
-                        PyObject* v = PyDict_GetItemString(kwds, "MAX_EVAL");
-                        if (!v) {
-                            PyErr_Clear();
-                            params->setAttributeValue("MAX_EVAL", (size_t)100*budget);
-                        }
+                    PyObject* v = PyDict_GetItemString(kwds, "MAX_EVAL");
+                    if (!v) {
+                        PyErr_Clear();
+                        params->setAttributeValue<size_t>("MAX_EVAL", (size_t)100*budget);
                     }
+
+                } else if (strcmp(key, "MAX_EVAL") == 0) {
+                    size_t mxeval = PyLong_AsSize_t(PyTuple_GET_ITEM(pair, 1));
+                    if (mxeval == (size_t)-1 && PyErr_Occurred()) {
+                        options_ok[0] = false;
+                        break;
+                    }
+                    params->setAttributeValue<size_t>(key, (size_t)mxeval);
+
                 } else if (strcmp(key, "BB_OUTPUT_TYPE") == 0) {
                     const char* value = PyCompat_PyText_AsString(PyTuple_GET_ITEM(pair, 1));
                     if (!value) {
@@ -394,7 +402,8 @@ static PyObject* minimize(PyObject* /* dummy */, PyObject* args, PyObject* kwds)
                         break;
                     }
                     options_ok[1] = true;
-                    params->setAttributeValue("BB_OUTPUT_TYPE", stringToBBOutputTypeList(value));
+                    params->setAttributeValue(key, stringToBBOutputTypeList(value));
+
                 } else if (strcmp(key, "SEED") == 0) {
                     const char* value = PyCompat_PyText_AsString(PyTuple_GET_ITEM(pair, 1));
                     if (!value) {
@@ -404,10 +413,11 @@ static PyObject* minimize(PyObject* /* dummy */, PyObject* args, PyObject* kwds)
                              options_ok[0] = false;
                              break;
                         }
-                        params->setAttributeValue("SEED", (int)seed);
+                        params->setAttributeValue<int>(key, (int)seed);
                     } else
                         params->readParamLine(std::string(key) + " " + value);
                     options_ok[5] = true;
+
                 } else {
                     if (strcmp(key, "BB_OUTPUT_TYPE") == 0)        options_ok[1] = true;
                     else if (strcmp(key, "DISPLAY_DEGREE") == 0)   options_ok[2] = true;
