@@ -6,13 +6,14 @@
 /*                 Christophe Tribes           - Polytechnique Montreal            */
 /*                                                                                 */
 /*  The copyright of NOMAD - version 4.0.0 is owned by                             */
+/*                 Charles Audet               - Polytechnique Montreal            */
 /*                 Sebastien Le Digabel        - Polytechnique Montreal            */
 /*                 Viviane Rochon Montplaisir  - Polytechnique Montreal            */
 /*                 Christophe Tribes           - Polytechnique Montreal            */
 /*                                                                                 */
-/*  NOMAD v4 has been funded by Rio Tinto, Hydro-Québec, NSERC (Natural Science    */
-/*  and Engineering Research Council of Canada), INOVEE (Innovation en Energie     */
-/*  Electrique and IVADO (The Institute for Data Valorization)                     */
+/*  NOMAD v4 has been funded by Rio Tinto, Hydro-Québec, NSERC (Natural            */
+/*  Sciences and Engineering Research Council of Canada), InnovÉÉ (Innovation      */
+/*  en Énergie Électrique) and IVADO (The Institute for Data Valorization)         */
 /*                                                                                 */
 /*  NOMAD v3 was created and developed by Charles Audet, Sebastien Le Digabel,     */
 /*  Christophe Tribes and Viviane Rochon Montplaisir and was funded by AFOSR       */
@@ -26,8 +27,6 @@
 /*    Polytechnique Montreal - GERAD                                               */
 /*    C.P. 6079, Succ. Centre-ville, Montreal (Quebec) H3C 3A7 Canada              */
 /*    e-mail: nomad@gerad.ca                                                       */
-/*    phone : 1-514-340-6053 #6928                                                 */
-/*    fax   : 1-514-340-5665                                                       */
 /*                                                                                 */
 /*  This program is free software: you can redistribute it and/or modify it        */
 /*  under the terms of the GNU Lesser General Public License as published by       */
@@ -79,7 +78,7 @@ template<> bool NOMAD::StopReason<NOMAD::BaseStopType>::checkTerminate() const
             return false;
             break;
         default:
-            NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
+            throw NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
     }
     return false;
 }
@@ -115,7 +114,7 @@ template<> bool NOMAD::StopReason<NOMAD::MadsStopType>::checkTerminate() const
             return false;
             break;
         default:
-            NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
+            throw NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
     }
     return false;
 }
@@ -132,6 +131,18 @@ template<> std::map<NOMAD::PhaseOneStopType,std::string> & NOMAD::StopReason<NOM
 }
 
 
+// Dictionary function for SSDMadsStopType
+template<> std::map<NOMAD::SSDMadsStopType,std::string> & NOMAD::StopReason<NOMAD::SSDMadsStopType>::dict() const
+{
+    static std::map<NOMAD::SSDMadsStopType,std::string> dictionary = {
+        {NOMAD::SSDMadsStopType::STARTED,"Started"},   // Set a the begining of a Step
+        {NOMAD::SSDMadsStopType::X0_FAIL,"Problem with starting point evaluation"}
+        //{NOMAD::SSDMadsStopType::MADS_FAIL,"Mads has terminated but no feasible point obtained"}
+    };
+    return dictionary;
+}
+
+
 // Returns true only to terminate an algorithm using PhaseOne (Mads, ...)
 template<> bool NOMAD::StopReason<NOMAD::PhaseOneStopType>::checkTerminate() const
 {
@@ -145,11 +156,30 @@ template<> bool NOMAD::StopReason<NOMAD::PhaseOneStopType>::checkTerminate() con
             return false;
             break;
         default:
-            NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for terminate");
+            throw NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for terminate");
     }
     return false;
 }
 
+
+template<> bool NOMAD::StopReason<NOMAD::SSDMadsStopType>::checkTerminate() const
+{
+    switch ( _stopReason )
+    {
+        case NOMAD::SSDMadsStopType::X0_FAIL:
+        //case NOMAD::SSDMadsStopType::MADS_FAIL:
+            return true;
+            break;
+        case NOMAD::SSDMadsStopType::STARTED:
+            return false;
+            break;
+        default:
+            throw NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for terminate");
+    }
+    return false;
+}
+
+
 // Dictionary function for LHStopType
 template<> std::map<NOMAD::LHStopType,std::string> & NOMAD::StopReason<NOMAD::LHStopType>::dict() const
 {
@@ -175,7 +205,7 @@ template<> bool NOMAD::StopReason<NOMAD::LHStopType>::checkTerminate() const
             return false;
             break;
         default:
-            NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
+            throw NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
     }
     return false;
 }
@@ -226,7 +256,7 @@ template<> bool NOMAD::StopReason<NOMAD::NMStopType>::checkTerminate() const
             return false;
             break;
         default:
-            NOMAD::Exception ( __FILE__, __LINE__,"All NM stop types must be checked for algo terminate");
+            throw NOMAD::Exception ( __FILE__, __LINE__,"All NM stop types must be checked for algo terminate");
     }
     return false;
 }
@@ -257,90 +287,124 @@ template<> bool NOMAD::StopReason<NOMAD::IterStopType>::checkTerminate() const
             return false;
             break;
         default:
-            NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
+            throw NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
 
     }
     return false;
 }
 
-// Dictionary function for EvalStopType
-template<> std::map<NOMAD::EvalStopType,std::string> & NOMAD::StopReason<NOMAD::EvalStopType>::dict() const
+
+// Dictionary function for EvalGlobalStopType
+template<> std::map<NOMAD::EvalGlobalStopType,std::string> & NOMAD::StopReason<NOMAD::EvalGlobalStopType>::dict() const
 {
-    static std::map<NOMAD::EvalStopType,std::string> dictionary = {
-        {NOMAD::EvalStopType::STARTED,                  "Started"},   // Set a the begining of an EvaluatorControl Run
-        {NOMAD::EvalStopType::MAX_BB_EVAL_REACHED,      "Max number of blackbox evaluations"},
-        {NOMAD::EvalStopType::LAP_MAX_BB_EVAL_REACHED,  "Max number of blackbox evaluations for a sub algorithm run (lap run)"},
-        {NOMAD::EvalStopType::MAX_EVAL_REACHED,         "Max number of total evaluations"},
-        {NOMAD::EvalStopType::OPPORTUNISTIC_SUCCESS,    "Success found and opportunistic strategy is used"},
-        {NOMAD::EvalStopType::EMPTY_LIST_OF_POINTS,     "Tried to eval an empty list"},
-        {NOMAD::EvalStopType::ALL_POINTS_EVALUATED,     "No more points to evaluate"},
-        {NOMAD::EvalStopType::MAX_BLOCK_EVAL_REACHED,   "Maximum number of block eval reached"},
-        {NOMAD::EvalStopType::MAX_SGTE_EVAL_REACHED,    "Max number of surrogate evaluations reached"}
+    static std::map<NOMAD::EvalGlobalStopType,std::string> dictionary = {
+        {NOMAD::EvalGlobalStopType::STARTED,                  "Started"},   // Set a the begining of an EvaluatorControl Run
+        {NOMAD::EvalGlobalStopType::MAX_BB_EVAL_REACHED,      "Max number of blackbox evaluations"},
+        {NOMAD::EvalGlobalStopType::MAX_EVAL_REACHED,         "Max number of total evaluations"},
+        {NOMAD::EvalGlobalStopType::MAX_BLOCK_EVAL_REACHED,   "Maximum number of block eval reached"}
     };
     return dictionary;
 }
 
-template<> bool NOMAD::StopReason<NOMAD::EvalStopType>::checkTerminate() const
+
+// Dictionary function for EvalMainThreadStopType
+template<> std::map<NOMAD::EvalMainThreadStopType,std::string> & NOMAD::StopReason<NOMAD::EvalMainThreadStopType>::dict() const
+{
+    static std::map<NOMAD::EvalMainThreadStopType,std::string> dictionary = {
+        {NOMAD::EvalMainThreadStopType::STARTED,                  "Started"},   // Set a the begining of an EvaluatorControl Run
+        {NOMAD::EvalMainThreadStopType::LAP_MAX_BB_EVAL_REACHED,  "Max number of blackbox evaluations for a sub algorithm run (lap run)"},
+        {NOMAD::EvalMainThreadStopType::SUBPROBLEM_MAX_BB_EVAL_REACHED,  "Max number of blackbox evaluations for a subproblem run"},
+        {NOMAD::EvalMainThreadStopType::OPPORTUNISTIC_SUCCESS,    "Success found and opportunistic strategy is used"},
+        {NOMAD::EvalMainThreadStopType::EMPTY_LIST_OF_POINTS,     "Tried to eval an empty list"},
+        {NOMAD::EvalMainThreadStopType::ALL_POINTS_EVALUATED,     "No more points to evaluate"},
+        {NOMAD::EvalMainThreadStopType::MAX_SGTE_EVAL_REACHED,    "Max number of surrogate evaluations reached"}
+    };
+    return dictionary;
+}
+
+
+template<> bool NOMAD::StopReason<NOMAD::EvalGlobalStopType>::checkTerminate() const
 {
     // Returns true only to terminate an algorithm (Mads, ...)
     switch ( _stopReason )
     {
-        case NOMAD::EvalStopType::MAX_BB_EVAL_REACHED:
-        case NOMAD::EvalStopType::LAP_MAX_BB_EVAL_REACHED:
-        case NOMAD::EvalStopType::MAX_EVAL_REACHED:
-        case NOMAD::EvalStopType::MAX_BLOCK_EVAL_REACHED:
+        case NOMAD::EvalGlobalStopType::MAX_BB_EVAL_REACHED:
+        case NOMAD::EvalGlobalStopType::MAX_EVAL_REACHED:
+        case NOMAD::EvalGlobalStopType::MAX_BLOCK_EVAL_REACHED:
             return true;
             break;
-        case NOMAD::EvalStopType::STARTED:
-        case NOMAD::EvalStopType::OPPORTUNISTIC_SUCCESS:
-        case NOMAD::EvalStopType::EMPTY_LIST_OF_POINTS:
-        case NOMAD::EvalStopType::ALL_POINTS_EVALUATED:
-        case NOMAD::EvalStopType::MAX_SGTE_EVAL_REACHED:
+        case NOMAD::EvalGlobalStopType::STARTED:
             return false;
             break;
         default:
-            NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
+            throw NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
     }
     return false;
 }
 
 
-// Dictionary function for SgtelibModelStopType
-template<> std::map<NOMAD::SgtelibModelStopType,std::string> & NOMAD::StopReason<NOMAD::SgtelibModelStopType>::dict() const
+template<> bool NOMAD::StopReason<NOMAD::EvalMainThreadStopType>::checkTerminate() const
 {
-    static std::map<NOMAD::SgtelibModelStopType,std::string> dictionary = {
-        {NOMAD::SgtelibModelStopType::STARTED, "Started"},   // Set a the begining of a Step
-        {NOMAD::SgtelibModelStopType::ORACLE_FAIL, "Oracle failed generating points"},
+    switch ( _stopReason )
+    {
+        case NOMAD::EvalMainThreadStopType::LAP_MAX_BB_EVAL_REACHED:
+        case NOMAD::EvalMainThreadStopType::SUBPROBLEM_MAX_BB_EVAL_REACHED:
+        case NOMAD::EvalMainThreadStopType::MAX_SGTE_EVAL_REACHED:
+            return true;
+            break;
+        case NOMAD::EvalMainThreadStopType::STARTED:
+        case NOMAD::EvalMainThreadStopType::OPPORTUNISTIC_SUCCESS:
+        case NOMAD::EvalMainThreadStopType::EMPTY_LIST_OF_POINTS:
+        case NOMAD::EvalMainThreadStopType::ALL_POINTS_EVALUATED:
+            return false;
+            break;
+        default:
+            throw NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
+    }
+    return false;
+}
 
-        {NOMAD::SgtelibModelStopType::MODEL_OPTIMIZER_FAIL, "Model Optimizer has failed"},
-        {NOMAD::SgtelibModelStopType::NO_POINTS, "No points to build model"},
-        {NOMAD::SgtelibModelStopType::NO_NEW_POINTS_FOUND, "Models optimization did not find new points"},
-        {NOMAD::SgtelibModelStopType::EVAL_FAIL, "Problem with Sgtelib Model evaluation"},
-        {NOMAD::SgtelibModelStopType::X0_FAIL, "Problem with starting point evaluation"},
-        {NOMAD::SgtelibModelStopType::ALL_POINTS_EVALUATED,"No more points to evaluate"}
+
+// Dictionary function for ModelStopType
+template<> std::map<NOMAD::ModelStopType,std::string> & NOMAD::StopReason<NOMAD::ModelStopType>::dict() const
+{
+    static std::map<NOMAD::ModelStopType,std::string> dictionary = {
+        {NOMAD::ModelStopType::STARTED, "Started"},   // Set a the begining of a Step
+        {NOMAD::ModelStopType::ORACLE_FAIL, "Oracle failed generating points"},
+
+        {NOMAD::ModelStopType::MODEL_OPTIMIZATION_FAIL, "Model Optimization has failed"},
+        {NOMAD::ModelStopType::INITIAL_FAIL, "Cannot initialize model"},
+        {NOMAD::ModelStopType::NOT_ENOUGH_POINTS, "Not enough points to build model"},
+        {NOMAD::ModelStopType::NO_NEW_POINTS_FOUND, "Models optimization did not find new points"},
+        {NOMAD::ModelStopType::EVAL_FAIL, "Problem with Model evaluation"},
+        {NOMAD::ModelStopType::X0_FAIL, "Problem with starting point evaluation"},
+        {NOMAD::ModelStopType::ALL_POINTS_EVALUATED,"No more points to evaluate"},
+        {NOMAD::ModelStopType::MODEL_SINGLE_PASS_COMPLETED,"A single pass to create trial point has been completed successfully."}
     };
     return dictionary;
 }
 
 
-// Returns true only to terminate an SgtelibModel algorithm
-template<> bool NOMAD::StopReason<NOMAD::SgtelibModelStopType>::checkTerminate() const
+// Returns true only to terminate an model based algorithms (sgtelib, quad, ...)
+template<> bool NOMAD::StopReason<NOMAD::ModelStopType>::checkTerminate() const
 {
     switch ( _stopReason )
     {
-        case NOMAD::SgtelibModelStopType::STARTED:
-        case NOMAD::SgtelibModelStopType::ALL_POINTS_EVALUATED:
+        case NOMAD::ModelStopType::STARTED:
+        case NOMAD::ModelStopType::ALL_POINTS_EVALUATED:
+        case NOMAD::ModelStopType::MODEL_SINGLE_PASS_COMPLETED:
             return false;
             break;
-        case NOMAD::SgtelibModelStopType::MODEL_OPTIMIZER_FAIL:
-        case NOMAD::SgtelibModelStopType::NO_POINTS:
-        case NOMAD::SgtelibModelStopType::NO_NEW_POINTS_FOUND:
-        case NOMAD::SgtelibModelStopType::EVAL_FAIL:
-        case NOMAD::SgtelibModelStopType::X0_FAIL:
+        case NOMAD::ModelStopType::MODEL_OPTIMIZATION_FAIL:
+        case NOMAD::ModelStopType::INITIAL_FAIL:
+        case NOMAD::ModelStopType::NOT_ENOUGH_POINTS:
+        case NOMAD::ModelStopType::NO_NEW_POINTS_FOUND:
+        case NOMAD::ModelStopType::EVAL_FAIL:
+        case NOMAD::ModelStopType::X0_FAIL:
             return true;
             break;
         default:
-            NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
+            throw NOMAD::Exception ( __FILE__, __LINE__,"All stop types must be checked for algo terminate");
     }
     return false;
 }
